# -*- coding: utf-8 -*-
from typing import Optional
from spacy.language import Language

@Language.factory("stylecheck")
def get_stylecheck(nlp: Language, name: str):
    return Stylecheck(nlp, name)


class Stylecheck:
    def __init__(self, nlp: Language, name: str = "stylecheck"):

        """
        nlp: an instance of spacy
        """
        self.nlp = nlp
        self.name = "stylecheck"
        self.ruler = nlp.add_pipe("entity_ruler")
        for label in patterns:
            label_patterns = [{"label":label,"pattern":p} for p in patterns[label]]
            self.ruler.add_patterns(label_patterns)

    def __call__(self, doc):
        return doc

# list of verbs that their adjective form
# is sometimes mistaken as a verb
verbs_list = ["associate", "involve", "exhaust", "base", "lead", "stun", "overrate",  "fill", "bear", "complicate", "reserve", "complicate"]

patterns = {
    "PASSIVE":[
        [
            {"DEP": "auxpass"},
            {"DEP":"neg", "TAG":"RB", "OP":"*"},
            {"DEP":"HYPH", "OP":"*"},
            {"DEP":"advmod", "TAG":"RB", "OP":"*"},
            { "TAG":"VBN", "LEMMA":{"NOT_IN" : verbs_list}}
        ],
        #
        # sentence : The book was read by him.
        # dependencies : ['det', 'nsubjpass', 'auxpass', 'ROOT', 'agent', 'pobj', 'punct']
        # Tags : ['DT', 'NN', 'VBD', 'VBN', 'IN', 'PRP', '.']
        #
        [
            {"DEP": {"IN": ["attr", 'nsubjpass', 'appos']}},
            {"TAG": "RB", "DEP": "advmod", "OP" : "*"},
            {"DEP": "PUNCT", "OP" : "*"},
            {"TAG": "VBN", "DEP": "acl","LEMMA": {"NOT_IN" : verbs_list}}
        ],
        #
        # sentence : there was no change detected in her behavior.
        # dependencies : ['expl', 'ROOT', 'det', 'attr', 'acl', 'prep', 'poss', 'pobj', 'punct']
        # tags : ['EX', 'VBD', 'DT', 'NN', 'VBN', 'IN', 'PRP$', 'NN', '.']
        #
        [
            {"DEP":"cc"},
            {"DEP":"advmod", "TAG":"VBN", "OP": "*", "LEMMA": {"NOT_IN":["pre"]}},
            {"DEP": "conj", "TAG": "VBN", "LEMMA":{"NOT_IN" : verbs_list}},
            {"DEP":"pobj", "OP":"!"}
        ],
        #
        # Used for the second part with "and ..."
        # sentence : it was determined and formed.
        # dependencies : ['nsubjpass', 'auxpass', 'ROOT', 'cc', 'conj', 'punct']
        # tags : ['PRP', 'VBD', 'VBN', 'CC', 'VBN', '.']
        #
        # TODO: Investigate why this rule causes grief
        [
            {"DEP":"advcl", "TAG":"VBN"},
            {"DEP": "agent", "TAG":"IN"},
            {"OP":"*"},
            {"DEP": "pobj", "TAG":"NN"},
        ],
        #
        # sentence : killed by the police, he never thought this would be his end.
        # dependencies : ['advcl', 'agent', 'det', 'pobj', 'punct', 'nsubj', 'neg', 'ROOT', 'nsubj', 'aux', 'ccomp', 'poss', 'attr']
        # tags : ['VBN', 'IN', 'DT', 'NN', ',', 'PRP', 'RB', 'VBD', 'DT', 'MD', 'VB', 'PRP$', 'NN']
        #
        [
            {"DEP": "nsubj"},
            {"DEP": "ROOT"},
            {"DEP": "attr", "TAG": "VBN", "LEMMA":{"NOT_IN" : verbs_list}},
            {"DEP": "prep", "TAG": "IN", "OP":"*"}
        ],
        #
        # sentence : Bears are dreamt of in your fantasies!
        # dependencies : ['nsubjpass', 'auxpass', 'ROOT', 'prep', 'prep', 'poss', 'pobj', 'punct']
        # tags : ['NNS', 'VBP', 'VBN', 'IN', 'IN', 'PRP$', 'NNS', '.']
        #
        [
            {"LEMMA": {"IN": verbs_list}},
            {"LOWER":"by"}
        ],
        #
        # to avoid the confusion between the adjective and passive version of specific
        # verbs, we dedicated a new rule to some verbs to be detected when used with
        # an agent (by)
        # sentence : Natural resources are exhausted by humans.
        # dependencies : ['amod', 'nsubjpass', 'auxpass', 'ROOT', 'agent', 'pobj']
        # tags : ['JJ', 'NNS', 'VBP', 'VBN', 'IN', 'NNS']
        #
    ]
}
